﻿using System;
using System.ComponentModel;
using System.Linq;
using Telerik.Windows.Controls;

namespace AutocompleteBehavior
{
	public class CollectionViewProvider : IAutocompleteProvider
	{
		private ICollectionView collectionView;

		public CollectionViewProvider(ICollectionView collectionView)
		{
			this.collectionView = collectionView;
		}

		public event ItemsReceivedEventHandler ItemsReceived;

		public TextSearchMode TextSearchMode { get; set; }

		public string TextPath { get; set; }

		Func<object, object> getText;

		public void GetItems(string text)
		{
			var match = TextSearch.CreatePartialMatchFunc(text, this.TextSearchMode);
			if (this.collectionView.CanFilter)
			{
				this.collectionView.Filter = (o) =>
					{
						return match(o as string);
					};
				this.ItemsReceived(this, new ItemsReceivedEventArgs(this.collectionView.OfType<string>().Take(20)));
			}
			else
			{
				if (getText == null)
				{
					var originalCollection = this.collectionView.SourceCollection.OfType<object>();
					var firstItem = originalCollection.FirstOrDefault();
					if (firstItem != null)
					{
						getText = BindingExpressionHelper.CreateGetValueFunc(firstItem.GetType(), this.TextPath);
					}
				}

				this.ItemsReceived(this, new ItemsReceivedEventArgs(this.collectionView.OfType<object>().Where((o) =>
					{
						var t = getText(o);
						return match(t as string);
					}).Take(20)));
			}
		}
	}
}