﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Xml.Linq;
using Telerik.Windows.Controls;
using AutocompleteBehavior;

namespace AutocompleteBox
{
	public class GoogleAutocompleteProvider : IAutocompleteProvider
	{
#if SILVERLIGHT
		private const string AutocompleteUrl = "../search.ashx?client=ie8&q=";
#else
		private const string AutocompleteUrl = "http://clients5.google.com/complete/search?hl=en-us&client=ie8&q=";
#endif
		private WebClient client = new WebClient();

		public event ItemsReceivedEventHandler ItemsReceived;

		public TextSearchMode TextSearchMode { get; set; }

		public string TextPath { get; set; }

		public void GetItems(string text)
		{
			if (this.client.IsBusy)
			{
				DownloadStringCompletedEventHandler retry = null;
				retry = (s, e) =>
					{
						this.client.DownloadStringCompleted -= retry;
						if (e.Cancelled)
						{
							this.GetItems(text);
						}
					};
				this.client.DownloadStringCompleted += retry;
				this.client.CancelAsync();
			}
			else
			{
				DownloadStringCompletedEventHandler getValues = null;
				getValues = (s, e) =>
				{
					this.client.DownloadStringCompleted -= getValues;
					if (!e.Cancelled)
					{
						this.OnItemsReceived(this.ParseResult(e.Result));
					}
				};
				this.client.DownloadStringCompleted += getValues;
				this.client.DownloadStringAsync(new Uri(AutocompleteUrl + text, UriKind.RelativeOrAbsolute));
			}
		}

		private IList<string> ParseResult(string result)
		{
			var xml = XDocument.Parse((string)result);

			var values = new List<string>();
			foreach (var node in xml.Descendants(XName.Get("Text", "http://opensearch.org/searchsuggest2")))
			{
				values.Add(node.Value);
			}

			return values;
		}

		protected virtual void OnItemsReceived(IList<string> results)
		{
			if (this.ItemsReceived != null)
			{
				this.ItemsReceived(this, new ItemsReceivedEventArgs(results));
			}
		}
	}
}